package nu.zoom.dsl.ast;

import nu.zoom.dsl.parser.StatesBaseVisitor;
import nu.zoom.dsl.parser.StatesParser;

import java.util.*;

public class StatesVisitorTransformer extends StatesBaseVisitor<StatesParser.DocumentContext> {
    private final HashMap<String,String> config = new HashMap<>();
    private final HashSet<TypeNode> nodeTypes = new HashSet<>();
    private final HashSet<TypeNode> messageTypes = new HashSet<>();
    // from -> <to, message>
    private final HashMap<String,HashMap<String, String>> transitions = new HashMap<>();

    @Override
    public StatesParser.DocumentContext visitTransition(StatesParser.TransitionContext ctx) {
        String from = ctx.from().state().typeName().IDENTIFIER().getText() ;
        String to = ctx.to().state().typeName().IDENTIFIER().getText() ;
        String message = ctx.message().typeName().IDENTIFIER().getText() ;
        this.transitions.computeIfAbsent(from, k -> new HashMap<>()).put(to, message);
        return super.visitTransition(ctx);
    }

    @Override
    public StatesParser.DocumentContext visitState(StatesParser.StateContext ctx) {
        String stateName = ctx.typeName().IDENTIFIER().getText() ;
        List<FieldNode> fields = extractFields(ctx.typeDeclaration()) ;
        this.nodeTypes.add(new TypeNode(stateName, fields));
        return super.visitState(ctx);
    }

    public Set<StateNode> getStates() {
        // TODO: Calculate state nodes from this.transitions
        return Set.of();
    }

    public Map<String,String> getConfig() {
        return Map.copyOf(config);
    }

    public Set<TypeNode> getTypes() {
        // TODO calculate data types from NodeTypes and MessageTypes with duplicate check.
        return Set.of() ;
    }

    private List<FieldNode> extractFields(StatesParser.TypeDeclarationContext declaration) {
        return declaration
                .typeField()
                .stream()
                .map(
                        ctx ->
                                new FieldNode(ctx.fieldName().getText(), ctx.fieldType().getText())
                )
                .toList();
    }
}
